/**
 * The main class of this library.
 *
 * {@link TextWrapper#constructor Create an instance} (with or without {@link WrapOptions options}) and then use
 * {@linkcode TextWrapper#wrap wrap()} method to do the task.
 *
 * Created at 1398/2/6 (2019/4/26).
 * @author {@link https://mirismaili.github.io S. Mahdi Mir-Ismaili}
 *
 * @see {@link TextWrapper#constructor The constructor}
 * @see {@linkcode TextWrapper#wrap wrap()}
 */
export default class TextWrapper implements WrapOptions {
    private readonly tabLength_1;
    readonly continuationIndent: string;
    readonly tabLength: number;
    readonly wrapOn: number;
    readonly allowedExceedingCharacters: RegExp;
    readonly breakableCharacters: RegExp;
    /**
     * @see {@linkcode DEF_WRAP_OPTIONS}
     */
    constructor(wrapOptions?: WrapOptions);
    /**
     * The main process in this library is done by this method. Gets a string and wraps illegal-too-long-lines of it. To
     * configure and customize its operation pass a {@linkcode WrapOptions} to the class constructor}.
     * Note: This is an *instance method* and needs to a an {@link TextWrapper#constructor object instantiation} first.
     *
     * @param text The string that probably has long lines and must be wrapped (broken to limited-length lines) based on
     * {@linkcode WrapOptions} that has been passed to the {@link TextWrapper#constructor class constructor}
     *
     * @param alreadyPresentIndentation If your input text has a fixed indentation in the leading of its lines already,
     * pass it (as a string) to this parameter. <br>
     * ***Important note***: The library doesn't check this *indentation* actually presents or not.
     * It just appends this to the {@linkcode WrapOptions.continuationIndent} (that you passed to the
     * {@link TextWrapper#constructor class constructor}).
     * In other words, (for a single run of this method per an instantiation) there is no difference whether you pass:
     * <br>
     * **`s1` to {@link WrapOptions.continuationIndent `WrapOptions.continuationIndent`}** and **`s2` to this parameter**
     * <br>or: <br>
     * **`s3` to {@link WrapOptions.continuationIndent `WrapOptions.continuationIndent`}** and **`s4` to this parameter**
     * <br>
     * **while `(s1 + s2 === s3 + s4)`**. The library works with *the whole* always.
     * In these cases, this is just a logical separation (and maybe usable in the future).
     *
     * @return {WrapResult} Two things that will be stored in a {@linkcode WrapResult}:
     * 1. {@linkcode WrapResult.wrappedText wrappedText: string} *(probably you want this)*
     * 2. {@linkcode WrapResult.markers markers: number[]}
     *
     * @see {@linkcode WrapResult}
     * @see {@linkcode TextWrapper#constructor class constructor}
     */
    wrap(text: string, alreadyPresentIndentation?: string): WrapResult;
    getVisualLength(s: string, visualOffset?: number): number;
    debug: {
        enable: (namespaces: string) => void;
        disable: () => string;
        setLog: (log: (...args: any[]) => any) => (...args: any[]) => any;
    };
}
/**
 * The return value of the main operation of this library ({@link TextWrapper#wrap wrapping texts}) would be of this
 * type.
 * Although the final output will be stored in {@linkcode wrappedText} and you regularly need this,
 * but there is also a more advanced output: {@linkcode markers}.
 */
export interface WrapResult {
    /**
     * The final output. This would be as same as the your original input,
     * but in addition to ***a few break-line characters* + *possibly indentations***
     * (**`'\n' + calculated_indentations`**).
     *
     * @see <code>alreadyPresentIndentation</code> in {@linkcode TextWrapper#wrap#alreadyPresentIndentation wrap()}
     *
     * @see WrapOptions.continuationIndent
     */
    wrappedText: string;
    /**
     * Each member of this array would point to a location that a *line-break* (possibly: *line-break + indentation*)
     * is needed.
     *
     * So you can reproduce the final output ({@linkcode wrappedText}) completely using
     * ***this array* + *your original input***.
     */
    markers: number[];
}
/**
 * Preferences to pass to the {@link TextWrapper} constructor to control its behavior.
 * All choices are optional. See default values in {@link DEF_WRAP_OPTIONS}.
 */
export interface WrapOptions {
    /**
     * This is the most useful option. It determines the maximum allowed length of each line. The words exceed this rule
     * will go to the next line. There are some configurable options to say the program how does this work:
     * 1. Allowed characters for exceeding the limitation (by default: white-spaces): {@link allowedExceedingCharacters}
     * 2. Word-boundaries: {@link breakableCharacters}
     * 3. Tab-character length: {@link tabLength}
     * 4. The indents to be inserted in the leading of each newly created line: {@link continuationIndent}
     */
    wrapOn?: number;
    /**
     * This determines the max-length that should be considered for tab-characters (`'\t'`). Notice that the length of
     * each tab-character is depended on its location in its line. For example if this option is set to `4`, then the
     * length of `'\tA'` will be appeared `5` and the length of `'A\t'` will be appeared `4`.
     */
    tabLength?: number;
    /**
     * Long lines don't break at any where the length is exceeded. But it occurs only on word-boundaries by default.
     * So this options has been set to `/[^\w\xA0]/` to be broken on any non-word character (`/^\w/ === /[^a-zA-Z0-9_]/`)
     * except a special white-space character named *non-breakable space (`'\xA0'`)*.
     *
     * Note: This RegExp will be tested against only a single character (each time)!
     */
    breakableCharacters?: RegExp;
    /**
     * This determines which characters are allowed to exceed the limitation (after {@link wrapOn} value). By default,
     * this has been set to `/\s/` to allow any white-space character. new-line-character (`'\n'`) will be ignored,
     * anyway.
     *
     * Note: This RegExp will be tested against only a single character (each time)!
     */
    allowedExceedingCharacters?: RegExp;
    /**
     * This value will be added after each line-break (`'\n'`) character that this tool adds to the text. So it appears
     * in the leading of each new line (not the already-present lines).
     */
    continuationIndent?: string;
}
export declare const DEF_WRAP_OPTIONS: WrapOptions;
export declare function wrapper(text: string, wrapOptions?: WrapOptions, alreadyPresentIndentation?: string): string;
